/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_OMNETPP_NODE_PROCESSOR_ABSTRACTNODEPROCESSOR_H_INCLUDED
#define SSTMAC_BACKENDS_OMNETPP_NODE_PROCESSOR_ABSTRACTNODEPROCESSOR_H_INCLUDED

#include <omnetpp.h>

#include <sstmac/software/ami/environment.h>

#include <omnetppaddress.h>
#include <node_m.h>

#include <sstmac/common/debug.h>

#include <sstmac/common/sim_parameters.h>

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/software/process/softwareid.h>

#include <sstmac/hardware/network/congestion/interconnect.h>
#include <sstmac/hardware/network/topology/fake_topology.h>

#include <sstmac/common/ptr_type.h>

#include <sstmac/hardware/node/node.h>
#include <sstmac/common/eventmanager.h>
#include <node/processor/eventhandlerpod.h>
#include <fake_eventmanager.h>

namespace sstmac
{
  namespace omnetpp
  {

    //class Application;

#define APP_STACK_SIZE 16384

#define OS_MSG_HEADER_SIZE 8
#define APP_SIZE 1000

#define APP_START_DELAY 0.00001

#define SEND_DELAY 0.00001

    class abstractnodeprocessor : public cSimpleModule
    {

      friend class fakeeventmanager;

    protected:



      class fakeinterconnect : public sstmac::hw::interconnect
      {
      public:
        typedef boost::intrusive_ptr<fakeinterconnect> ptr;
        typedef boost::intrusive_ptr<const fakeinterconnect> const_ptr;

        virtual std::string
        to_string() const
        {
          return "fakeinterconnect";
        }

        virtual
        ~fakeinterconnect()
        {

        }

        // Optional finish function for children
        virtual void
        finish()
        {
        }

        /// The maximum number of nodes on this network.
        /// Return a negative (really <= 0) value to indicate an infinite capacity.
        virtual int64_t
        slots() const;

        /// Send the given message at the given time.
        /// Notify the sendhandler when the send is complete.
        /// Notify the recvhandler when the recv is complete.
        virtual void
        send(const timestamp &start_time, const sst_message::ptr &msg);

      //  virtual sstmac::hw::topology::ptr
      //  get_topology();

        static ptr
        construct(abstractnodeprocessor* parent, sstmac::hw::topology::ptr top)
        {
          return ptr(new fakeinterconnect(parent, top));
        }

      protected:
        abstractnodeprocessor* parent_;
        fakeinterconnect(abstractnodeprocessor* parent, sstmac::hw::topology::ptr top) :
          interconnect(top)
        {
          parent_ = parent;
        }
      };

    public:
      abstractnodeprocessor();
      virtual
      ~abstractnodeprocessor();

      omnetpp_address::ptr
      get_addr()
      {
        return myid;
      }

      simtime_t
      opp_simtime()
      {
        return simTime();
      }

    protected:

      virtual void
      handleMessage(cMessage* cmg);
      virtual void
      initialize();
      virtual void
      finish();

      std::list<ptr_type::ptr> finishes_;

      omnetpp_address::ptr myid;
      int coresPerProc, procsPerNode, numOfNodes;

      int nodesPerRack;
      int numRacks;

      cGate* outgate;
      cGate* ingate;

      sstmac::hw::node::ptr node_;

      static debug dbg_;

      boost::intrusive_ptr<fakeeventmanager> fem_;

      virtual sstmac::timestamp
      now();

      // ------------------------------------ //

    };

  }
}
#endif
