#include <sstmac/common/resource/resource.h>

namespace sstmac {

debug resource::dbg_; std::list<resource::ptr> resource::all_resources_;

resource::resource(int nmax)
    : ndelays_(0), total_delay_(0), nactive_(0), nmax_(nmax)
{
    if (!dbg_.is_real())
    {
        dbg_ = debug("<debug> resource");
        logger::register_logger(&dbg_);
    }

    all_resources_.push_back(resource::ptr(this));
}

resource::~resource()
{
}

int
resource::use()
{
    ++nactive_;
    return nactive_;
}

void
resource::register_resource(const resource::ptr &res)
{
    all_resources_.push_back(res);
}

int
resource::nmax() const
{
    return nmax_;
}

void
resource::handle(const sst_message::ptr &msg)
{
    throw illformederror("resource shouldn't actually be used as eventhandler");
}

void
resource::add_event(const eventcallback::ptr &event)
{
    pending_events_.push_back(event);
    SSTMAC_DEBUG << "resource: add event in pending queue for " << to_string() << ". "
                 << "now " << nactive_ << " active events"
                 << " and "<< pending_events_.size() << " events waiting\n";

    /** mark the callback for when the resource wanted to run it, but couldn't */
    event->set_time(now());
}

void
resource::release()
{
    SSTMAC_DEBUG << "resource: release resource " << to_string() << ": "
                 << "  nactive=" << nactive_
                 << "  npending=" << pending_events_.size()
                 << "\n";

    if (nactive_ == 0){
        throw illformederror("Resource already has zero events. Resource release error.");
    }
    --nactive_; //just release
    if (pending_events_.empty()){
        SSTMAC_DEBUG << "resource: no more pending events on " << to_string() << "\n";
    }
    else {
        SSTMAC_DEBUG << "resource: still has "
                     << pending_events_.size()
                     << " events pending. running next event\n";

        eventcallback::ptr cb = pending_events_.front();
        pending_events_.pop_front();
        ++ndelays_;
        timestamp delay = now() - cb->get_time();
        total_delay_ += delay;
        cb->callback(sst_message::ptr());
    }
}


}

