/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_SSTEVENT_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_SSTEVENT_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/common/malloc.h>

namespace sstmac
{
    class event :
            public ptr_type
    {

    protected:
        event();

    public:
      typedef boost::intrusive_ptr<event> ptr;

      virtual ~event();

      const timestamp& time() const;

      void set_time(const timestamp& t);

      event::ptr next;

      virtual void cancel() = 0;

      virtual bool is_cancelled() const = 0;

      virtual void execute() = 0;

      eventmodulePtr get_module() const;

      void set_module(const eventmodulePtr& mod);

    protected:
      eventmodulePtr module_;

      timestamp time_;

    };

    class simple_event :
            public event
    {
        protected:
            simple_event(const sst_message::ptr &msg,
                    const eventhandler::ptr &hand);

            sst_message::ptr msg_to_deliver_;

            eventhandler::ptr handler_;

            bool canceled_;
        public:
            typedef boost::intrusive_ptr<simple_event> ptr;

            ~simple_event();

            static ptr
            construct(const sst_message::ptr &msg,
                    const eventhandler::ptr &handler);

            virtual std::string to_string() const;

            void execute();

            void cancel();

            virtual bool is_cancelled() const {
              return canceled_;
            }

    };

} // end of namespace sstmac

#endif
