/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstream>

#include <sstmac/hardware/network/interconnect/multinetwork.h>
#include <sstmac/hardware/network/interconnect/divider.h>
#include <sstmac/common/driver_util.h>
#include <sstmac/common/prefix_params.h>

namespace sstmac
{
  namespace hw
  {

    SSTRegister("multi | multinetwork", interconnect, multinetwork);

    multinetwork::multinetwork(const topology::ptr& top, const sim_parameters::ptr& params) :
      interconnect(fake_topology::construct())
    {
      int num = 1;
      std::stringstream pre;
      pre << "multi" << num << "_";
      prefix_params::ptr par = prefix_params::construct(params, pre.str());
      std::string name = pre.str() + "network_name";
      bool go = params->has_param(name);
      while (go)
        {
          SSTMAC_DEBUG << "multi-network: constructing a network at index " << num << ": " << params->get_param(pre.str() + "network_name") << "\n";
          interconnect::ptr net = interconnect_factory::get_param(name, par);
          networks_[num] = net;
          num++;
          pre.clear();
          pre.str(std::string());
          pre << "multi" << num << "_";
          name = pre.str() + "network_name";
          SSTMAC_DEBUG << "multi-network: checking for " << name << "\n";
          go = params->has_param(name);
          par->set_prefix(pre.str());
        }
      num--;


    }

    /// Send the given message at the given time.
    /// Notify the sender node when the send is complete
    /// and notify the receiver node when receive needs to be performed.
    void
    multinetwork::send(const timestamp &start_time,
        const sst_message::ptr &msg)
    {
      /*int n = div_->which(msg);
      SSTMAC_DEBUG << "multi-network: sending message " << msg->to_string() << " to network " << n << "\n";
      if(networks_.find(n) == networks_.end()){
          std::map::iterator<int, interconnect::ptr>:: it, end = networks_.end();
          for(it = networks_.begin(); it != end; it++){
              std::cerr << "at index: " << it->first << " we have " << it->second->to_string() << "\n";
          }
          throw ssterror("multi-network: no network at index");
      }
      networks_[n]->send(start_time, msg, sendhandler, recvhandler);*/
      throw ssterror("multinetwork::send -- this function should not be called");
    }

    void
    multinetwork::set_eventmanager(const eventmanager::ptr &m)
    {
      eventmodule::set_eventmanager(m);

      boost::unordered_map<int, interconnect::ptr>::iterator it, end = networks_.end();
      for (it = networks_.begin(); it != end; it++)
        {
          it->second->set_eventmanager(m);
        }
    }

    void
    multinetwork::finish()
    {
      boost::unordered_map<int, interconnect::ptr>::iterator it, end = networks_.end();
      for (it = networks_.begin(); it != end; it++)
        {
          it->second->finish();
        }
    }

  }
}
