#ifndef TRAININTERCONNECT_H
#define TRAININTERCONNECT_H

/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_TRAIN_TRAININTERCONNECT_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TRAIN_TRAININTERCONNECT_H_INCLUDED

#include <sstmac/hardware/network/interconnect/interconnect.h>
#include <sstmac/hardware/network/topology/switchtopology.h>
//#include <sstmac/hardware/switch/trainswitch.h>
#include <boost/unordered_map.hpp>
#include <sstmac/common/driver_util.h>

namespace sstmac
{
  namespace hw
  {

    class traininterconnect : public interconnect, public vis::vis_display
    {

    protected:

      traininterconnect(const topology::ptr& top,
          const vis::vis_engine::ptr& veng) :
        interconnect(top), vis_display("interconnect", veng)
      {
      }

      void
      init(const sim_parameters::ptr& params);

      switchtopology::trainmap switches_;
      switchtopology::ptr switchtop_;
      int nps_;

    public:
      typedef boost::intrusive_ptr<traininterconnect> ptr;

      virtual std::string
      to_string() const
      {
        return "switchinterconnect";
      }

      static ptr
      construct(const sim_parameters::ptr& params);

      virtual
      ~traininterconnect();

      virtual void
      finish();

      virtual void
      set_endpoint(const nodeaddress::ptr& addr, const eventhandler::ptr& nic);

      /// Send the given message at the given time.
      /// Notify the sender node when the send is complete
      /// and notify the receiver node when receive needs to be performed.
      virtual void
      send(const timestamp &start_time, const sst_message::ptr &msg);

      virtual void
      set_eventmanager(const eventmanager::ptr &m);

      virtual std::string
      label(long node_id) const;

      virtual vis::vis_topology::vismaptype
      vis_get_nodes();

      virtual vis::vis_topology::ptr
      vis_get_topology();

      virtual int64_t
      num_slots() const
      {
        return switchtop_->topology_size() * nps_;
      }

    };

  }
}

#endif

#endif // TRAININTERCONNECT_H
