/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_CYCLEACCURATEMESSAGE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_CYCLEACCURATEMESSAGE_H_INCLUDED

#include <sstmac/hardware/network/switches/packet/packetmessage.h>

namespace sstmac
{
  namespace hw
  {
    class cycleaccuratemessage : public packetmessage
    {
    public:
      enum MSG_TYPE {DATA, ACK};

    protected:
      cycleaccuratemessage(sst_message::ptr orig, long size, bool is_tail, int vc, MSG_TYPE t = DATA) :
        sst_message(NETWORK),
        packetmessage(orig, size, is_tail),
        vc_(vc), my_type_(t)
      {

      }



    public:
      typedef boost::intrusive_ptr<cycleaccuratemessage> ptr;

      static ptr
      construct(sst_message::ptr orig, long size, bool is_tail, int vc)
      {
        return ptr(new cycleaccuratemessage(orig, size, is_tail, vc));
      }

      static ptr
      construct(long size, int vc){
        return ptr(new cycleaccuratemessage(sst_message::ptr(), size, true, vc, ACK));
      }


      virtual std::string
      to_string() const
      {
        return "cycleaccuratemessage(" + (orig_ ? orig_->to_string() : "null msg") + ")";
      }

      virtual
      ~cycleaccuratemessage()
      {
      }

      int vc_;
      int inport_;
      MSG_TYPE my_type_;

      timestamp bufferarrive_;

      networkswitch::ptr prevswitch_;
      int prevport_;
    };
  }
}

#endif
