/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWTICHES_PACKET_PACKET_ROUTING_INFO_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWTICHES_PACKET_PACKET_ROUTING_INFO_H_INCLUDED

#include <sstmac/hardware/network/topology/routing_info.h>
#include <boost/unordered_map.hpp>

namespace sstmac
{
  namespace hw
  {

  typedef std::vector<timestamp> finish_time_vector;
  class packet_finish {
    public:
        packet_finish(int redund)
            : times(redund, timestamp(0)),
            round_robin_index(0)
        {
        }

        packet_finish()
            : times(0, timestamp(0)),
              round_robin_index(0)
        {
        }

        void increment_index(){
            round_robin_index = (round_robin_index + 1) % times.size();
        }

        finish_time_vector times;
        int round_robin_index;
  };

  //     outdim          outdir        redund   time
  typedef boost::unordered_map<int, packet_finish> outdir_to_finish_map;
  typedef boost::unordered_map<int, outdir_to_finish_map> outdim_to_finish_map;

    class packet_routing_info : public congestion_routing_info
    {
    protected:
      //      dim            dir   relative
      nested_type<int, int, double>::unordered_map congestion_;

      packet_routing_info(nodeaddress::ptr to) :
        congestion_routing_info(to)
      {

      }

    public:
      typedef boost::intrusive_ptr<packet_routing_info> ptr;

      static ptr
      construct(nodeaddress::ptr to)
      {
        return ptr(new packet_routing_info(to));
      }

      virtual
      ~packet_routing_info()
      {
      }

      virtual std::string
      to_string() const
      {
        return "packet_routing_info";
      }

      void
      set_congestion(
          const outdim_to_finish_map& finishtimes,
          timestamp maxtime, timestamp now);

      virtual void
      get_min_congestion_port(int &dim, int &dir);

      virtual nested_type<int, int, double>::unordered_map
      get_congestion_info()
      {
        return congestion_;
      }

    };

  }
}
#endif
