/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_PACKETMESSAGE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_PACKETMESSAGE_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/hardware/network/switches/packet/packet_routing_info.h>

namespace sstmac
{
  namespace hw
  {
    class packetmessage :
        public network_message
    {


    protected:
      packetmessage(sst_message::ptr orig, long size, bool is_tail) :
        sst_message(sst_message::NETWORK), 
        orig_(orig), size_(size),
            tail_(is_tail)
      {
        rinfo_ = packet_routing_info::construct(dest());
      }

      sst_message::ptr orig_;
      long size_;
      bool tail_;
      packet_routing_info::ptr rinfo_;

    public:

      typedef boost::intrusive_ptr<packetmessage> ptr;

      static ptr
      construct(sst_message::ptr orig, long size, bool is_tail)
      {
        return ptr(new packetmessage(orig, size, is_tail));
      }

      virtual long
      get_byte_length() const
      {
        return size_;
      }

      virtual nodeaddress::ptr
      dest()
      {
        if(orig_)
          return orig_->toaddr();
        else
          return nodeaddress::ptr();
          // sst_throw(ssterror, "packetmessage - can't get dest addr");
      }

      packet_routing_info::ptr
      rinfo()
      {
        return rinfo_;
      }

      virtual bool
      is_tail() const
      {
        return tail_;
      }

      virtual sst_message::ptr
      orig() const
      {
        return orig_;
      }

      sst_message::ptr
      get_parent_msg() const
      {
          return orig_;
      }

      virtual std::string
      to_string() const
      {
        return "packetmessage";
      }

      virtual
      ~packetmessage()
      {
      }
    };
  }
}

#endif
