/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// butterfly.h: Interface for butterfly networks.
//
// Based on butterfly.c by Jim Schutt
// Adapted by Curtis Janssen <cljanss@ca.sandia.gov>
#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_butterfly_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_butterfly_H_INCLUDED

#include <sstmac/hardware/network/topology/switchtopology.h>

namespace sstmac
{
  namespace hw
  {

    class butterfly : public switchtopology
    {
    private:
      int n_;
      int r_;

      int stages_;

    protected:
      //butterfly(TiXmlHandle);
      butterfly(int n, int r, int nps);

    public:
      typedef boost::intrusive_ptr<butterfly> ptr;

      static butterfly::ptr
      construct(sim_parameters::ptr params);

      virtual
      ~butterfly()
      {
      }

      virtual int64_t
      n() const
      {
        return n_;
      }

      int r() const {
          return r_;
      }

      virtual int64_t
      topology_size() const
      {
        return n();
      }

      int64_t
      diameter() const
      {
        return 1;
      }

      virtual int
      radix() const
      {
        return r_;
      }

      virtual void
      connect_switches(switchtopology::maptype &switches,
                       const std::vector<int>& red,
                       double linkbw);

      virtual void
      set_ejectors(long num, int nps, const maptype& switches, ejmaptype &ejectors,
          const sim_parameters::ptr& params);

      virtual std::string
      to_string() const
      {
        return "butterfly";
      }
    };

  }
} //end of namespace sstmac

#endif
