/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/network/topology/butterflyrouter.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac
{
  namespace hw
  {

    SSTRegister("butterfly | bfly", router, butterflyrouter);

    void
    butterflyrouter::route(routing_info::ptr to, int &dim, int &dir)
    {
      nodeaddress::ptr netaddr = convert_to_network_addr(to->to());

      SSTMAC_DEBUG << "butterflyrouter(" << me_->to_string()
            << "): heading to " << to->to()->to_string() << " at net addr "
            << netaddr->to_string() << "\n";

      if (me_->unique_id() >= (stages_ - 1) * numperstage_) //i'm an output stage
        {
          dim = networkswitch::EJECT_PORT;
          dir = to->to()->unique_id() % conc_;
        }
      else
        {
          dim = 0;
          int myid = me_->unique_id();
          if (netaddr->unique_id() == myid % numperstage_)
            { //we're going straight
              SSTMAC_DEBUG << "going straight \n";
              dir = 0;
              return;
            }
          else
            {
              int stage = myid / numperstage_;
              int skip = pow(r_, stage);
              long mylogid = myid % (numperstage_);
              long diff = (netaddr->unique_id() - mylogid);
             // dir = (myid / skip + (netaddr->unique_id() / skip)) % r_;
              dir = (diff / skip + numperstage_) % r_;
              SSTMAC_DEBUG << "stage " << stage << ", skip " << skip
                    << ", numperstage " << numperstage_ << ", mylogid " << mylogid << ", diff " << diff << ", dir " << dir
                    << "\n";
              return;
            }
        }
      return;

    }

    int
    butterflyrouter::convert_to_outport(int dim, int dir)
    {
      if (dim == networkswitch::EJECT_PORT)
        {
          return -(1+dir);
        }
      return dir;
    }

    void
    butterflyrouter::convert_to_dimdir(int outport, int &dim, int&dir)
    {
      if (outport < 0)
        {
          dim = networkswitch::EJECT_PORT;
          dir = (-outport) - 1;
        }
      else
        {
          dim = 0;
          dir = outport;
        }
    }

    int
    butterflyrouter::get_inport(nodeaddress::ptr from, int dim, int dir)
    {
      if(inports_.find(from->unique_id()) == inports_.end()){
          inports_[from->unique_id()] = incnt_++;
      }

      return inports_[from->unique_id()];

    }

  }
}
