/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#include <sstmac/hardware/network/topology/crossbarrouter.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac
{
  namespace hw
  {

    SSTRegister("crossbar | xbar", router, crossbarrouter);

    void
    crossbarrouter::route(routing_info::ptr to, int &dim, int &dir)
    {
      nodeaddress::ptr netaddr = convert_to_network_addr(to->to());
      if (me_->equals(netaddr))
        {
          dim = networkswitch::EJECT_PORT;
          dir = to->to()->unique_id() % conc_;
          SSTMAC_DEBUG << "crossbarrouter(" << me_->to_string()
                << "): returning ejection port \n";
        }
      else
        {
          dim = 0;
          dir = netaddr->unique_id();
          SSTMAC_DEBUG << "crossbarrouter(" << me_->to_string()
                << "): returning dim 0 and dir " << dir << " \n";
        }
    }

    int
    crossbarrouter::convert_to_outport(int dim, int dir)
    {
      if(dim == networkswitch::EJECT_PORT){
          return -(1+dir);
      }
      return dir;
    }

    void
    crossbarrouter::convert_to_dimdir(int outport, int &dim, int&dir)
    {
      if (outport < 0)
        {
          dim = networkswitch::EJECT_PORT;
          dir = (-outport) - 1;
        }
      else
        {
          dim = 0;
          dir = outport;
        }
    }

    int
    crossbarrouter::get_inport(nodeaddress::ptr from, int dim, int dir)
    {
      return from->unique_id();
    }

  }
}
