/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_ROUTING_CROSSBARROUTER_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_ROUTING_CROSSBARROUTER_H_INCLUDED

#include <sstmac/hardware/network/topology/router.h>
#include <sstmac/hardware/network/topology/crossbar.h>

namespace sstmac
{
  namespace hw
  {

    class crossbarrouter : public router
    {
    protected:

    public:
      typedef boost::intrusive_ptr<crossbarrouter> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        return new crossbarrouter;
      }

      void init(const nodeaddress::ptr& addr, const topology::ptr& top)
      {
          crossbar::ptr crsbar = safe_cast<crossbar>(top);
          router::init(crsbar->nps(), addr);
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir);

      virtual
      ~crossbarrouter()
      {
      }

      virtual std::string
      to_string() const
      {
        return "crossbarrouter(" + me_->to_string() + ")";
      }

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

    protected:

    };

  }
}
#endif
