/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// hdtorus.h: Interface for torus networks.
//
// Author: Ali Pinar <apinar@sandia.gov>

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_DRAGONFLY_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_DRAGONFLY_H_INCLUDED

#include <sstmac/hardware/network/topology/switchtopology.h>

namespace sstmac
{
  namespace hw
  {

    class dragonfly : public switchtopology
    {

    protected:
      dragonfly(long x, long y, long g, int gc, int nps);

      int x_;
      int y_;
      int g_;
      int group_con_;

    public:
      typedef boost::intrusive_ptr<dragonfly> ptr;

      static ptr
      construct(sim_parameters::ptr params);

      virtual std::string
      to_string() const
      {
        return "dragonfly";
      }

      int
      numX() const
      {
        return x_;
      }

      int
      numY() const
      {
        return y_;
      }

      int
      numG() const
      {
        return g_;
      }

      int
      group_con() const {
        return group_con_;
      }

      void
      get_coords(long uid, int &x, int &y, int &g);

      long
      get_uid(int x, int y, int g);

      virtual int64_t
      n() const
      {
        return x_ * y_ * g_;
      }

      virtual std::string
      label(long id);

      virtual
      ~dragonfly()
      {
      }

      virtual int64_t
      topology_size() const
      {
        return n();
      }

      virtual int
      radix() const
      {
        return x_ + y_ + group_con_;
      }

      std::string
      coord_string(int x, int y, int g);

      virtual void
      connect_switches(switchtopology::maptype &switches,
          const std::vector<int>& red, double linkbw);

      void
      connect_train_tracks(trainmap& switches, const std::vector<int>& red,
          double bytes_per_cycle, double freq_hz, timestamp link_latency,
          double payload_fraction, bool ignore_congestion,
          bool enable_multipath, const output_bw_arbitrator::ptr& arb_template);

      virtual int64_t
      diameter() const
      {
        return -1;
      }

    };

  }
} //end of namespace sstmac

#endif
