/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/network/topology/dragonflyrouter_nonminimal.h>
#include <math.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac
{
  namespace hw
  {
    using namespace sstmac;
    using namespace sstmac::hw;
    using namespace std;

    SSTRegister("dragonfly_nonminimal | dfly_nonminimal", router, dragonflyrouter_nonminimal);

    void
    dragonflyrouter_nonminimal::route(routing_info::ptr to, int &dim, int &dir)
    {
      nodeaddress::ptr netaddr = convert_to_network_addr(to->to());
      int hisx, hisy, hisg;
      get_coords(netaddr->unique_id(), hisx, hisy, hisg);
      SSTMAC_DEBUG << "cray_dragon_router_nm(" << me_->to_string()
            << "): heading to " << to->to()->to_string() << " at net addr "
            << netaddr->to_string() << "...";

      if (hisg != myG_)
      { //route to the right group first
        route_group(to, dim, dir);
      }
      else if (hisx == myX_ && hisy == myY_)
      {
        SSTMAC_DEBUG << "ejecting \n";
        dim = networkswitch::EJECT_PORT;
        dir = to->to()->unique_id() % conc_;
      }
      else
      {
        if (to->is_min()) //we haven't routed randomly yet
        {
          SSTMAC_DEBUG << "routing randomly...";

          dim = rng_->value() % 2;
          if (dim == 0)
          {

            dir = myX_;
            while (dir == myX_)
            {
              dir = rng_->value() % numX_;
            }
            SSTMAC_DEBUG << "in x: " << dir << "\n";
          }
          else
          {
            dir = myY_;
            while (dir == myY_)
            {
              dir = rng_->value() % numY_;
            }
            SSTMAC_DEBUG << "in y: " << dir << "\n";
          }

          to->set_min(false);
        }
        else
        {
          SSTMAC_DEBUG << "routing minimally...";

          if (myX_ == hisx)
          {
            dim = 1;
            dir = hisy;

            SSTMAC_DEBUG << "in y: " << dir << "\n";
          }
          else if (myY_ == hisy)
          {
            dim = 0;
            dir = hisx;

            SSTMAC_DEBUG << "in x: " << dir << "\n";
          }
          else //not at either, pick randomly
          {
            dim = rng_->value() % 2;
            if (dim == 0)
            {

              dir = hisx;

              SSTMAC_DEBUG << "in x: " << dir << "\n";
            }
            else
            {
              dir = hisy;

              SSTMAC_DEBUG << "in y: " << dir << "\n";
            }
          }

        }

      }

    }

  }

}
