/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_ROUTING_DRAGONFLYROUTER_UGAL_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_ROUTING_DRAGONFLYROUTER_UGAL_H_INCLUDED

#include <sstmac/hardware/network/topology/dragonflyrouter.h>

namespace sstmac
{
  namespace hw
  {

    class dragonflyrouter_ugal : public dragonflyrouter
    {

    protected:
      double switch_lat_;
      double link_lat_;
      dragonflyrouter_ugal(double lnk, double sw) :
        link_lat_(lnk), switch_lat_(sw)
      {

      }

      virtual bool
      needs_congestion_info() const
      {
        return true;
      }

    public:
      typedef boost::intrusive_ptr<dragonflyrouter_ugal> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        double switch_lat = 0;
        double link_lat = 0;
        if (params->has_param("network_train_switch_latency_cycles"))
        {
          switch_lat = params->get_int_param(
              "network_train_switch_latency_cycles");
          double freq = params->get_freq_param("network_train_switch_freq");
          switch_lat = switch_lat / (1e6 * freq);
          link_lat = params->get_int_param("network_train_link_latency");
        }

        return ptr(new dragonflyrouter_ugal(link_lat, switch_lat));
      }

      virtual void
      route(sstmac::hw::routing_info::ptr to, int &dim, int &dir);

      virtual
      ~dragonflyrouter_ugal()
      {
      }

    };

  }
}
#endif
