/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_FATTREEROUTER_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_FATTREEROUTER_H_INCLUDED

#include <sstmac/hardware/network/topology/router.h>
#include <sstmac/common/messages/sst_message.h>
#include <math.h>
#include <sstmac/hardware/network/switches/circuit/circuit_router.h>
#include <sstmac/hardware/network/topology/fattree.h>

namespace sstmac
{
  namespace hw
  {

    class fattreerouter : public virtual router, public virtual circuit_router
    {
    public:
      typedef boost::intrusive_ptr<fattreerouter> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        return new fattreerouter;
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual
      ~fattreerouter()
      {
      }

      virtual void
      init(const nodeaddress::ptr& me, const topology::ptr& top)
      {
          fattree::ptr ft_top = safe_cast<fattree>(top);
          k_ = ft_top->k();
          l_ = ft_top->l();

          router::init(ft_top->nps(), me);

        int switchesperlevel = pow(k_, l_ - 1);
        myL_ = me->unique_id() / switchesperlevel;
        logicalid_ = (me->unique_id() - (myL_ * switchesperlevel)) / pow(k_,
            myL_);
        downwid_ = 1;
      }

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

      virtual void
      path_is_good(nodeaddress::ptr goingto, int fromport, int toport)
      {
      }

      virtual void
      path_teardown(int fromport, int toport)
      {
      }

      virtual std::string
      to_string() const
      {
        return "fattreerouter";
      }

      virtual void
      set_inport(nodeaddress::ptr from, int port);

    protected:
      int l_;
      int k_;

      int myL_;
      int logicalid_;
      int downwid_;

      std::map<long, int> inports_;


    };

  }
}
#endif
