/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/network/topology/flattenedbflyrouter.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac
{
  namespace hw
  {

    SSTRegister("fbfly | flattenedbfly", router, flattenedbflyrouter);

    void
    flattenedbflyrouter::route(routing_info::ptr to, int &dim, int &dir)
    {
      nodeaddress::ptr netaddr = convert_to_network_addr(to->to());
      std::vector<int> coords = get_coords(netaddr->unique_id());
      SSTMAC_DEBUG << "flattenedbflyrouter(" << me_->to_string() << "): heading to "
            << to->to()->to_string() << " at net addr " << netaddr->to_string()
            << "\n";

      for (int i = 0; i < (int) dims_.size(); i++)
        {
          if (coords[i] != coords_[i])
            {
              dim = i;
            }
          else
            {
              continue;
            }

          dir = coords[i];
          SSTMAC_DEBUG << "returning dim " << dim << " and dir " << dir << "\n";
          return;

        }

      dim = networkswitch::EJECT_PORT;
      dir = to->to()->unique_id() % conc_;
      return;

    }

    int
    flattenedbflyrouter::convert_to_outport(int dim, int dir)
    {

      if (dim == networkswitch::EJECT_PORT)
        {
          return -(1+dir);
        }
      long ret = 0;
      for (int i = 0; i < (int) dims_.size() && i < dim; i++)
        {
          ret += dims_[i];

        }
      ret += dir;

      return ret;
    }

    void
    flattenedbflyrouter::convert_to_dimdir(int outport, int &dim, int&dir)
    {

      if (outport < 0)
        {
          dim = networkswitch::EJECT_PORT;
          dir = (-outport)-1;
        }
      else
        {
          long temp = outport;
          for (int i = 0; i < (int) dims_.size(); i++)
            {
              if (dims_[i] > temp)
                {
                  dim = i;
                  dir = temp;
                  break;
                }
              else
                {
                  temp -= dims_[i];
                }

            }

        }
    }

    int
    flattenedbflyrouter::get_inport(nodeaddress::ptr from, int dim, int dir)
    {
      std::vector<int> coords = get_coords(from->unique_id());
      return convert_to_outport(dim, coords[dim]);

    }

  }
}
