/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// flattenedbutterfly.h: Interface for torus networks.
//
// Author: Ali Pinar <apinar@sandia.gov>

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_flattenedbutterfly_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_flattenedbutterfly_H_INCLUDED

#include <sstmac/hardware/network/topology/switchtopology.h>

namespace sstmac
{
  namespace hw
  {

    class flattenedbutterfly : public switchtopology
    {

    protected:

      int nps_;
      std::vector<int> dimensions_;

      flattenedbutterfly(const std::vector<int>& dimensionarray, int nndpersw);

    public:
      typedef boost::intrusive_ptr<flattenedbutterfly> ptr;

      static flattenedbutterfly::ptr
      construct(sim_parameters::ptr params);

      std::vector<int>
      get_coords(long i);

      virtual int
      radix() const
      {
        return 2 * dimensions_.size();
      }

      long
      get_netid(const std::vector<int>& v);

      std::string
      coord_string(const std::vector<int>& coords);

      virtual
      ~flattenedbutterfly()
      {
      }

      int64_t
      diameter() const
      {
        return dimensions_.size();
      }

      virtual int64_t
      topology_size() const;

      /// Returns the vector giving each dimension of the torus.
      const std::vector<int> &
      dimensions() const
      {
        return dimensions_;
      }

      virtual void
      display_nodes(vismaptype &switches, const vis::vis_engine::ptr& eng,
          std::list<vis::vis_obj::ptr> &objs);

      virtual void
      connect_switches(switchtopology::maptype &switches,
                       const std::vector<int>& red,
                       double linkbw);

      int ndimensions() const {
          return dimensions_.size();
      }

      virtual std::string
      to_string() const
      {
        return "flattenedbutterflytopology";
      }
    };

  }
} //end of namespace sstmac

#endif
