/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// hdtorus.h: Interface for torus networks.
//
// Author: Ali Pinar <apinar@sandia.gov>

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_GROUPED_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_GROUPED_H_INCLUDED

#include <sstmac/hardware/network/topology/switchtopology.h>
#include <sstmac/hardware/network/topology/groupedrouter.h>

namespace sstmac
{
  namespace hw
  {

    class grouped : public switchtopology
    {

    protected:
      grouped(const switchtopology::ptr& inner, const switchtopology::ptr& outer, long g,
          int nps);

      switchtopology::ptr inner_;
      switchtopology::ptr outer_;
      int g_;
      int nps_;

      sim_parameters::ptr inpar_;
      sim_parameters::ptr outpar_;

    public:
      typedef boost::intrusive_ptr<grouped> ptr;

      static ptr
      construct(const sim_parameters::ptr& params);

      virtual std::string
      to_string() const
      {
        return "grouped";
      }

      void
      get_coords(long uid, int &x, int &y, int &g);

      long
      get_netid(int x, int y, int g);

      virtual int64_t
      n() const
      {
        return inner_->n() * g_;
      }

      virtual
      ~grouped()
      {
      }

      virtual int
      radix() const
      {
        return inner_->radix() + outer_->radix();
      }

      virtual void
      connect_switches(switchtopology::maptype &switches,
                       const std::vector<int>& red,
                       double linkbw);

      virtual router::ptr
      create_router(const nodeaddress::ptr& addr, const sim_parameters::ptr& params);

      virtual int64_t
      diameter() const
      {
        return -1;
      }

      virtual int64_t
      topology_size() const
      {
        return n();
      }

      class superswitch : public networkswitch
      {

      protected:
        switchtopology::maptype mysw_;

        superswitch(switchtopology::maptype sw);

        virtual injector::ptr
        get_new_injector(sim_parameters::ptr params);

        virtual ejector::ptr
        get_new_ejector(sim_parameters::ptr params);

      public:
        typedef boost::intrusive_ptr<superswitch> ptr;

        static ptr
        construct(switchtopology::maptype sw)
        {
          return ptr(new superswitch(sw));
        }

        virtual
        ~superswitch()
        {
        }

        virtual void
        connect(int dim, int dir, networkswitch::ptr sw, double bw, int redund = 1);

        virtual void
        initialize(sim_parameters::ptr params, int nps)
        {
        }

        virtual networkswitch::ptr
        clone(router::ptr route, nodeaddress::ptr addr);

      };

    };

  }
} //end of namespace sstmac

#endif
