/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/network/topology/groupedrouter.h>
#include <sstmac/hardware/network/switches/networkswitch.h>

namespace sstmac
{
  namespace hw
  {

    int groupedrouter::GROUP_DIM = 100;

    void
    groupedrouter::route(routing_info::ptr to, int &dim, int &dir)
    {
      nodeaddress::ptr netaddr = convert_to_network_addr(to->to());

      SSTMAC_DEBUG << "groupedrouter(" << me_->to_string() << "): heading to "
            << to->to()->to_string() << " at net addr " << netaddr->to_string()
            << "\n";

      long hisg = netaddr->unique_id() / innercount_;

      if (me_->unique_id() == netaddr->unique_id()) //i'm there man
        {
          dim = networkswitch::EJECT_PORT;
          dir = to->to()->unique_id() % conc_;
        }
      else if (hisg != myG_)
        {
          SSTMAC_DEBUG << "groupedrouter(" << me_->to_string()
                << "): routing in outer network \n";

          int fakedim;
          outer_->route(routing_info::construct(nodeaddress::convert_from_uniqueid(hisg)), fakedim, dir);
          if (dir != me_->unique_id() % innercount_) // must route to switch that is connected to other group
            {
              SSTMAC_DEBUG << "groupedrouter(" << me_->to_string()
                    << "): need to go to switch " << dir << " first \n";
              inner_->route(routing_info::construct(nodeaddress::convert_from_uniqueid(dir)), dim, dir);
            }
          else
            {
              SSTMAC_DEBUG << "groupedrouter(" << me_->to_string()
                    << "): exiting out of this group \n";
              dim = GROUP_DIM + fakedim;
            }
        }
      else
        {
          SSTMAC_DEBUG << "groupedrouter(" << me_->to_string()
                << "): routing in inner network \n";

          nodeaddress::ptr tog = nodeaddress::convert_from_uniqueid(
              to->to()->unique_id() % (innercount_ * conc_));
          inner_->route(routing_info::construct(tog), dim, dir);
        }
      return;

    }

    int
    groupedrouter::convert_to_outport(int dim, int dir)
    {
      if (dim == networkswitch::EJECT_PORT)
        {
          return -(1+dir);
        }
      else if (dim >= GROUP_DIM)
        {
          return innerports_ + outer_->convert_to_outport(dim - GROUP_DIM, dir);
        }
      else
        {
          return inner_->convert_to_outport(dim, dir);
        }
    }

    void
    groupedrouter::convert_to_dimdir(int outport, int &dim, int&dir)
    {
      if (outport < 0)
        {
          dim = networkswitch::EJECT_PORT;
          dir = (-outport)-1;
        }
      else if (outport >= innerports_)
        {
          int fakedim;
          outer_->convert_to_dimdir(outport - innerports_, fakedim, dir);
          dim = GROUP_DIM + fakedim;
        }
      else
        {
          inner_->convert_to_dimdir(outport, dim, dir);
        }
    }

    int
    groupedrouter::get_inport(nodeaddress::ptr from, int dim, int dir)
    {
      if (dim >= GROUP_DIM)
        {
          return innerports_ + outer_->get_inport(
              nodeaddress::convert_from_uniqueid(
                  from->unique_id() / innercount_), dim - GROUP_DIM, dir);
        }
      else
        {
          return inner_->get_inport(
              nodeaddress::convert_from_uniqueid(
                  from->unique_id() % innercount_), dim, dir);
        }

    }

  }
}
