/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_groupedrouter_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWTICHES_ROUTING_groupedrouter_H_INCLUDED

#include <sstmac/hardware/network/topology/router.h>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/hardware/network/switches/circuit/circuit_router.h>

namespace sstmac
{
  namespace hw
  {

    class groupedrouter : public virtual router
    {
    public:
      typedef boost::intrusive_ptr<groupedrouter> ptr;

      static ptr
      construct(int r, router::ptr innerroute, router::ptr outerroute,
          long innercount, int inneroutports, int g)
      {
        return ptr(
            new groupedrouter(r, innerroute, outerroute, innercount,
                inneroutports, g));
      }

      virtual void
      route(routing_info::ptr to, int &dim, int &dir);

      virtual int
      convert_to_outport(int dim, int dir);

      virtual void
      convert_to_dimdir(int outport, int &dim, int&dir);

      virtual
      ~groupedrouter()
      {
      }

      void init(const nodeaddress::ptr& me, const topology::ptr& top)
      {
          sst_throw(illformederror, "groupedrouter should never be constructed and inited from sim_params");
      }

      virtual void
      init(nodeaddress::ptr me)
      {
        router::init(conc_, me);
        myG_ = me->unique_id() / (innercount_);
      }

      static int GROUP_DIM;

      virtual int
      get_inport(nodeaddress::ptr from, int dim, int dir);

      virtual bool
      is_fixed_routing() const
      {
        return inner_->is_fixed_routing();
      }

      virtual std::string
      to_string() const
      {
        return "groupedrouter";
      }

    protected:
      int g_;
      int myG_;
      long innercount_;
      int innerports_;

      router::ptr inner_;
      router::ptr outer_;

      groupedrouter(int r, router::ptr innerroute, router::ptr outerroute,
          long innercount, int inneroutports, int g) :
        g_(g), innercount_(innercount),
          innerports_(inneroutports),
            inner_(innerroute), outer_(outerroute)
      {
        conc_ = r;
      }

    };

  }
}
#endif
