#include <sstmac/hardware/network/topology/switchtopology.h>
#include <sstmac/hardware/network/topology/router.h>

namespace sstmac {
    namespace hw {

void
switchtopology::set_ejectors(long num, int nps, const maptype& switches, ejmaptype &ejectors,
    const sim_parameters::ptr& params)
{
  for (long i = 0; i < num * nps; i++)
    {

      nodeaddress::ptr nodeaddr = nodeaddress::convert_from_uniqueid(i);

      nodeaddress::ptr swaddr = nodeaddress::convert_from_uniqueid(
          (i / nps));

      ejectors[nodeaddr] = mapget(switches,swaddr,"switch address")->get_ejector(params, i % nps);
      // HACK need an ejector if EJECT is used
      // (should EJECT be used?, does this work?)
      //ejectors[nodeaddr] = mapget(switches,swaddr,"switch address")->get_ejector(params, router::EJECT);

    }
}

switchtopology::switchtopology(int nps)
    : nps_(nps)
{
}

void
switchtopology::connect_train_tracks(trainmap &switches,
                                     const std::vector<int>& red,
                                     double bytes_per_cycle,
                                     double freq_hz,
                                     timestamp link_latency,
                                     double payload_fraction,
                                     bool ignore_congestion,
                                     bool enable_multipath,
                                     const output_bw_arbitrator::ptr &arb_template)
{
    sst_throw(unimplementederror, "given topology type does not support packet train models");
}

std::vector<int>
switchtopology::get_coords(long uid) const
{
    sst_throw(unimplementederror, "switchtopology::get_coords for %s", to_string().c_str());
}

long
switchtopology::get_netid(const std::vector<int>& coords) const
{
    sst_throw(unimplementederror, "switchtopology::get_uid for %s", to_string().c_str());
}

router::ptr
switchtopology::create_router(const nodeaddress::ptr &addr, const sim_parameters::ptr &params)
{
    router::ptr ret = router_factory::get_optional_param("router", factory_name_,
                                                   params, addr, ptr(this));
    return ret;
}

long
switchtopology::get_nodeid(const std::vector<int>& coords) const
{
    long net_id = get_netid(coords);
    int lastidx = coords.size() - 1;
    long nid = nps_ * net_id + coords[lastidx];
    return nid;
}

  }
}
