/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_SWITCHES_SWITCHTOPOLOGY_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_SWITCHES_SWITCHTOPOLOGY_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/sstmac-config.h>
#include <sstmac/hardware/network/switches/networkswitch.h>
#include <sstmac/hardware/packettrain/trainswitch.h>
#include <sstmac/hardware/network/switches/entryexit.h>
#include <sstmac/hardware/network/topology/router.h>
#include <sstmac/hardware/network/topology/topology.h>
#include <sstmac/common/vis/vis.h>


namespace sstmac
{
  namespace hw
  {


    /**
     * This class is an interface for topologies which support
     * the switch network model
     */
    class switchtopology : public topology, public virtual vis::vis_topology
    {

    protected:
        int nps_;

        std::string name_;

        switchtopology(int nps);

    public:  
      typedef boost::intrusive_ptr<switchtopology> ptr;

      typedef boost::unordered_map<nodeaddress::ptr, networkswitch::ptr> maptype;
      typedef boost::unordered_map<nodeaddress::ptr, trainswitch::ptr> trainmap;
      typedef boost::unordered_map<nodeaddress::ptr, injector::ptr> injmaptype;
      typedef boost::unordered_map<nodeaddress::ptr, ejector::ptr> ejmaptype;

      virtual void
      connect_switches(maptype &switches,
                       const std::vector<int>& red,
                       double linkbw) = 0;

      virtual router::ptr
      create_router(const nodeaddress::ptr& addr, const sim_parameters::ptr& params);

      virtual int
      radix() const = 0;

      virtual int64_t
      n() const
      {
        return topology_size();
      }

      int nps() const {
        return nps_;
      }

      virtual int ndimensions() const {
          return 0;
      }

      virtual std::vector<int>
      get_coords(long uid) const;

      virtual long
      get_netid(const std::vector<int>& coords) const;

      virtual long
      get_nodeid(const std::vector<int>& coords) const;

      virtual int64_t
      topology_size() const = 0;

      virtual void
      set_ejectors(long num, int nps, const maptype& switches, ejmaptype &ejectors,
          const sim_parameters::ptr& params);

      virtual void
      display_nodes(const vismaptype &switches, const vis::vis_engine::ptr& eng,
          std::list<vis::vis_obj::ptr> &objs)
      {

      }

      virtual void
      connect_train_tracks(trainmap& switches,
                           const std::vector<int>& red,
                           double bytes_per_cycle,
                           double freq_hz,
                           timestamp link_latency,
                           double payload_fraction,
                           bool ignore_congestion,
                           bool enable_multipath,
                           const output_bw_arbitrator::ptr& arb_template);
      virtual
      ~switchtopology()
      {
      }

      virtual std::string
      to_string() const
      {
        return "switchtopology";
      }
    };
  }
}

#endif
