/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

// Author: Curtis Janssen <cljanss@ca.sandia.gov>

#include <sstream>
#include <stdexcept>
#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac { namespace hw  {

  debug topology::dbg_;

  ImplementFactory(topology);

  std::string
  topology::name() const
  {
    std::ostringstream ostr;
    ostr << "Unknown(" << n() << "," << ")";
    return ostr.str();
  }

  std::string
  topology::label(long id)
  {
    return "vertex " + to_string() + sstprintf("2%d", id);
  }

  // Converts a node vertex to its index.
  int64_t
  topology::node_to_index(const vertex &v) const
  {
    std::map<vertex, int64_t>::const_iterator i = nodesi_.find(v);
    if (i == nodesi_.end())
      {
        i = nodesi_.begin();
        if (v.v().size() != i->first.v().size()){
            std::cerr << "vertex size: " << v.v().size() << ", nodes size: "
                      << i->first.v().size() << std::endl;
          throw invalidkeyerror(
              "network::node_to_index: vertex sizes don't match");
        }
        else{
            std::cerr << "vertex not found\n";
            v.print(std::cerr);
            std::cerr << std::endl;
            std::cerr << "Are you sure that specified network_geomtry "
                      << "is consistent with the node map?" << std::endl;
            throw std::runtime_error(
              "network::node_to_index: vertex not found");
        }
      }
    return i->second;
  }

} } //end of namespace sstmac
