/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NIC_GEMININIC_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NIC_GEMININIC_H_INCLUDED


#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/common/resource/resource.h>
#include <sstmac/software/libraries/compute/lib_compute_memmove.h>
#include <sstmac/hardware/network/interconnect/interconnect.h>

namespace sstmac { namespace hw
  {

  class rdma_nic:
    public networkinterface
  {

  protected:

    rdma_nic();

  public:

    typedef boost::intrusive_ptr<rdma_nic> ptr;

    static rdma_nic::ptr
    construct(
        const sim_parameters::ptr& params);

    /// Goodbye.
    virtual ~rdma_nic() throw ();

    virtual networkinterface::ptr
    clone(const node::ptr &parent,
        const sstmac::hw::interconnect::ptr &interconn,
        const sim_parameters::ptr& params) const;

    virtual void send(
        const nodeaddress::ptr &recver,
        const sst_message::ptr &payload
    );

    void set_eventmanager(const eventmanager::ptr &m);

    virtual void handle(const sst_message::ptr &msg);

    std::string
    to_string() const {
        return "rdma_nic";
    }

    virtual void recv(const sst_message::ptr& msg);

    virtual void notify_session_manager(const sst_message::ptr& msg);

    virtual void validate_msg_supported(const sst_message::ptr& msg) const;

    virtual void send_rdma_rsp(const sst_message::ptr& msg);

    virtual void recv_rdma_rsp(const sst_message::ptr& msg);

  };


  } } // end of namespace sstmac.

#endif
