/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SSTMAC_H_INCLUDED
#define SSTMAC_SSTMAC_H_INCLUDED

#include <sstmac/backends/native/driver.h>
#include <sstmac/common/sim_parameters.h>

#define PARSE_OPT_SUCCESS 0
#define PARSE_OPT_EXIT_SUCCESS 1
#define PARSE_OPT_EXIT_FAIL 2

struct opts
{
  int help;
  std::string debug;
  std::string configfile;
  sstmac::timestamp stoptime;
  sstmac::command_parameters::ptr params;
  std::string mode;
  bool color;
  std::string dumpimetafile;
  std::string network;
  bool print_apps;
  bool print_walltime;
  bool print_params;

  opts() :
      help(0), debug(""), configfile(""), stoptime(-1), mode("normal"), color(false),
      dumpimetafile(""), network(""), print_apps(false), print_walltime(true), print_params(false)
  {
  }
};

int
parse_opts(int argc, char **argv, opts &oo);
double
get_positive_float(const char *value, bool &errorflag);
int
get_int(const char *value, bool &errorflag);
int
get_positive_int(const char *value, bool &errorflag); std::vector<int>
get_intvec(const char *value, bool &errorflag);

void
print_help(int argc, char **argv);

void
run_debugger(sstmac::sim_parameters::ptr params, opts oo);

void
multisim_init(sstmac::sim_parameters::ptr params,
    std::map<int, std::string> &keys,
    std::map<int, std::vector<std::string> > &runs, bool print);



#endif
