/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <iostream>
#include <sstmac/software/process/app.h>
#include <sstmac/common/messages/eventdata.h>

using namespace sstmac;
using namespace sstmac::sw;

app::ptr
get_current_app()
{
  thread::ptr t = OS_API::get_current_thread();
  app::ptr app;
  try
    {
      app = boost::dynamic_pointer_cast<sstmac::sw::app>(t);
    }
  catch (std::exception& e)
    {
      std::cerr << e.what() << std::endl;
      return app::ptr();
    }

  if (!app)
    {
      throw sstmac::ssterror("sstcpp::get_current_app - app didn't cast");
    }

  return app;
}

extern "C" void
SSTMAC_compute(double seconds)
{
  app::ptr app = get_current_app();
  app->compute(timestamp(seconds));
}

extern "C" void
SSTMAC_compute_block_read(long long bytes)
{
  app::ptr app = get_current_app();
  app->compute_block_read(bytes);
}

extern "C" void
SSTMAC_compute_loop(int from, int to, int numlines)
{
  app::ptr app = get_current_app();
  lib_compute_loops::ptr lib = app->get_compute_loops_lib();
  lib->compute_loop(from, to, numlines, app->tid().id);
}

extern "C" void
SSTMAC_compute_loop2(int from1, int to1, int from2, int to2, int numlines)
{
  app::ptr app = get_current_app();
  lib_compute_loops::ptr lib = app->get_compute_loops_lib();
  lib->compute_loop2(from1, to1, from2, to2, numlines, app->tid().id);
}

extern "C" void
SSTMAC_compute_loop3(int from1, int to1, int from2, int to2, int from3,
    int to3, int numlines)
{
  app::ptr app = get_current_app();
  lib_compute_loops::ptr lib = app->get_compute_loops_lib();
  lib->compute_loop3(from1, to1, from2, to2, from3, to3, numlines, app->tid().id);
}

extern "C" void
SSTMAC_compute_loop4(int from1, int to1, int from2, int to2, int from3,
    int to3, int from4, int to4, int numlines)
{
  app::ptr app = get_current_app();
  lib_compute_loops::ptr lib = app->get_compute_loops_lib();
  lib->compute_loop4(from1, to1, from2, to2, from3, to3, from4, to4, numlines, app->tid().id);
}
