/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_comm/mpigroup.h>

namespace sstmac
{
  namespace sw
  {
    long mpigroup::groupcounter_ = 0;

    mpigroup::mpigroup(tasklist::const_ptr p) :
      peers_(p)
    {
        if (peers_->size() == 0){
            sst_throw(valueerror, "cannot create an mpi group of size 0");
        }
    }

    taskid
    mpigroup::at(int rank)
    {
      if (peers_ && peers_->size() > 0) // not comm_world
        {
          return peers_->at(rank);
        }
      else
        {
          return taskid(rank);
        }
    }

    size_t
    mpigroup::size() const
    {
      if (peers_ && peers_->size() > 0) // not comm_world
        {
          return peers_->size();
        }
      else
        {
          return size_;

        }
    }

    tasklist::const_ptr
    mpigroup::list()
    {
      return peers_;
    }

    mpiid
    mpigroup::rank_of_task(taskid t)
    {
      if (peers_ && peers_->size() > 0)
        {
          for (uint i = 0; i < peers_->size(); i++)
            {
              if (peers_->at(i) == t)
                {
                  return mpiid(i);
                }
            }
          return mpiid(-1);
        }
      else
        {
          return mpiid(t.id);
        }
    }

  }

}
