#ifndef sstmac_software_process_FTQ_H
#define sstmac_software_process_FTQ_H

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/stats/stat_logger.h>
#include <sstmac/common/config.h>
#include <sstmac/software/process/key.h>
#include <stdint.h>

#ifdef HAVE_VTK
#include <vtkRenderWindow.h>
#include <vtkCamera.h>
#include <vtkActor.h>
#include <vtkRenderer.h>
#include <vtkMapper.h>
#include <vtkPolyDataAlgorithm.h>
#include <vtkTextActor.h>

#include <vtkCubeSource.h>
#include <vtkCylinderSource.h>
#include <vtkPlaneSource.h>
#include <vtkLineSource.h>

#include <vtkPolyDataMapper.h>
#include <vtkProperty.h>
#include <vtkPolyDataWriter.h>
#include <vtkXMLPolyDataWriter.h>
#include <vtkAppendPolyData.h>

#include <vtkMatrixToLinearTransform.h>
#include <vtkTransform.h>
#include <vtkCellData.h>
#include <vtkPointData.h>
#include <vtkSmartPointer.h>

#include <vtkChartXY.h>
#include <vtkPlotBar.h>
#include <vtkPlotPoints.h>
#include <vtkPlotStacked.h>
#include <vtkTable.h>
#include <vtkIntArray.h>
#include <vtkFloatArray.h>
#include <vtkLongArray.h>
#include <vtkContextView.h>
#include <vtkContextScene.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkRegressionTestImage.h>
#include <vtkNew.h>
#include <vtkColorSeries.h>
#include <vtkChartLegend.h>
#include <vtkLegendBoxActor.h>
#include <vtkAxis.h>

#define VTK_CREATE(type, name) \
  vtkSmartPointer<type> name = vtkSmartPointer<type>::New()
#endif

#include <string>
#include <stdlib.h>
#include <boost/lexical_cast.hpp>


/**
  "Fixed time quanta" collection of the amount of work done in particular time intervals
  over the course of the computation
*/
namespace sstmac {
    namespace sw {


    class FTQEpoch {
        protected:
            long long totals_[KEY_NUM_EVENT_TYPES];

        public:
            typedef boost::intrusive_ptr<FTQEpoch> ptr;

            FTQEpoch();

            void collect(key::event_type_t ty, long ticks);

            long long get_event_time(key::event_type_t ty) const;

    };

    struct EventNode
    {
        key::event_type_t ev_type;

        long ticks_begin;

        long ticks_end;

        EventNode* next;

    };

    class TaskFTQCalendar :
        public ptr_type,
        public FTQEpoch
    {
        protected:
            EventNode* head_;

            EventNode* tail_;

            long max_tick_;

        public:
            typedef boost::intrusive_ptr<TaskFTQCalendar> ptr;

            TaskFTQCalendar();

            virtual ~TaskFTQCalendar(){}

            void collect(key::event_type_t ty, long ticks_begin, long ticks);

            virtual std::string to_string() const { return "TaskFTQCalendar"; }

            void dump(std::ofstream& os);

    };

    class AppFTQCalendar :
        public TaskFTQCalendar
    {
        private:
            boost::unordered_map<int, TaskFTQCalendar::ptr> calendars_;
            boost::unordered_map<int, FTQEpoch*> thread_epochs_;
#ifdef HAVE_VTK
            std::map <int, vtkContextView*> view_thread;
            std::map <int, vtkChartXY*> chart_thread;
#endif

            FTQEpoch* epochs_;

            int aid_;

            int max_tid_;

            long num_epochs_;

            long max_epoch_;

            long min_event_size_;

            long num_ticks_epoch_;

            bool show_vis_;

            bool show_vis_thread_chart_;

            std::string show_vis_thread_chart_type_;

            bool show_vis_thread_chart_separate_;

            bool show_vis_summary_chart_;

            std::string appname_;

#ifdef HAVE_VTK
            vtkSmartPointer<vtkContextView> view;
            vtkSmartPointer<vtkChartXY> chart;
#endif

        public:
            typedef boost::intrusive_ptr<AppFTQCalendar> ptr;

            AppFTQCalendar(int aid, const std::string& appname, long num_epochs, long nticks_epoch, long min_event_size, bool show_vis,
            		bool show_vis_thread_chart, std::string show_vis_thread_chart_type,
            		bool show_vis_thread_chart_separate, bool show_vis_summary_chart);

            void dump();

            virtual ~AppFTQCalendar(){}

            void collect(key::event_type_t ty, int tid, long ticks_begin, long num_ticks);

            void display_charts(int tid, long max_epoch);

            virtual std::string to_string() const { return "AppFTQCalendar"; }
    };

    class   FTQCalendar :
        public stat_logger
    {
        private:
            boost::unordered_map<int, AppFTQCalendar::ptr> calendars_;

            long num_epochs_;

            long min_event_size_;

            long num_ticks_epoch_;

            bool show_vis_;

            bool show_vis_thread_chart_;

            std::string show_vis_thread_chart_type_;

            bool show_vis_thread_chart_separate_;

            bool show_vis_summary_chart_;

            FTQCalendar();

        public:
            typedef boost::intrusive_ptr<FTQCalendar> ptr;

            void init(long num_epochs, long nticks_per_epoch, long min_event_size, bool show_vis,
            		bool show_vis_thread_chart, std::string show_vis_thread_chart_type,
            		bool show_vis_thread_chart_separate, bool show_vis_summary_chart);

            virtual ~FTQCalendar(){}

            void collect(key::event_type_t ty, int aid, int tid, long ticks_begin, long num_ticks);

            AppFTQCalendar::ptr get_calendar(int aid) const;

            static ptr
            construct();

            void register_app(int aid, const std::string& appname);

            void simulation_finished(timestamp end);

            virtual std::string to_string() const { return "FTQCalendar"; }
    };

 }
}

#endif // FTQ_H
