/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/process/key.h>
#include <sstmac/common/alloc.h>
#include <iostream>
#include <stdlib.h>

namespace sstmac { namespace sw {
  
  // Pointer types.
  
  

  // The key objects are very short-lived, so we don't feel like
  // continually allocating and deallocating those bastards.
  namespace keypool {
    static std::vector<key*>* chunks = 0;
  } // end of namespace keypool.

key::key() :
    ev_type_(general)
{
}

key::key(event_type_t ty) :
    ev_type_(ty)
{
}

key::ptr key::construct() {
      return new key;
}

key::ptr key::construct(event_type_t ty) {
    return new key(ty);
}


key::~key(){
}

#define enumcase(x) case x: return #x;
const char*
key::tostr(event_type_t ty)
{
    switch(ty)
    {
        enumcase(general);
        enumcase(mpi);
        enumcase(compute);
        enumcase(sleep);
        enumcase(buffer);
    }
    return "unknown";
}

void* key::operator new(size_t size) {
  if(! keypool::chunks) {
    keypool::chunks = new std::vector<key*>;
    keypool::chunks->reserve(1024);
  }

  void* memptr;
  if(!keypool::chunks->empty()) {
    memptr = keypool::chunks->back();
    keypool::chunks->pop_back();
  }
  else{
      memptr = ::malloc(size);
  }
  return memptr;
}

void key::add_blocker(thread_data_t t)
{
    blocking_t::iterator it = blocking_.find(t);
    if (it != blocking_.end())
    {
        std::cerr << "thread already there: " << it->second->to_string() << "\n";
        std::cerr << "me: " << (long)(this) << "\n";
        throw illformederror("key is blocking the same thread twice");
    }
    blocking_.insert(t);
}

void key::remove_blocker(thread_data_t t)
{
    blocking_.erase(t);
}

void key::operator delete(void* ptr){
  keypool::chunks->push_back(static_cast<key*>(ptr));
}

void key::clear_statics(){
    if (keypool::chunks){
        for (int i=0; i < keypool::chunks->size(); ++i){
            ::free(keypool::chunks->at(i));
        }
        keypool::chunks->clear();
        delete keypool::chunks;
    }
}

} } // end of namespace sstmac.
