/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_KEY_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_KEY_H_INCLUDED

#include <sstmac/software/threading/threading_interface.h>
#include <sstmac/software/process/thread.h>
#include <sstmac/common/ptr_type.h>
#include <cstring>

namespace sstmac { namespace sw {

  /**
   * A base type and default (empty) implementation of a handle
   * to block and unblock processes.
   */
  class key : public ptr_type {

  public:
    #define KEY_NUM_EVENT_TYPES 5
    typedef enum { compute=0, mpi=1, sleep=2, buffer=3, general=4 } event_type_t;
    // thread_data_t defined in process/thread.h
       typedef std::set<thread_data_t> blocking_t;
  private:
    friend class operatingsystem;



    blocking_t blocking_;

    event_type_t ev_type_;

    key();

    key(event_type_t);

  public:
    virtual
    std::string to_string() const {
        return "key";
    }

    typedef boost::intrusive_ptr<key> ptr;

    static const char* tostr(event_type_t ty);

    static key::ptr construct();

    static key::ptr construct(event_type_t ty);

    event_type_t event_type() const {
        return ev_type_;
    }

    void set_event(event_type_t ev) {
        ev_type_ = ev;
    }

    virtual ~key();

    void* operator new(size_t size);

    void operator delete(void* ptr);

    void
    add_blocker(thread_data_t t); 

    blocking_t
    get_blocking() {
      return blocking_;
    }

    void
    remove_blocker(thread_data_t t);

    bool
    still_blocked(){
      return blocking_.size() > 0;
    }

    void clear(){
      blocking_.clear();
    }

    static void clear_statics();

  };

} } // end of namespace sstmac

#endif
