#include <sstmac/software/services/service.h>
#include <sstmac/software/process/operatingsystem.h>
#include <sstmac/hardware/node/node.h>

namespace sstmac {
    namespace sw {


    ImplementFactory(service);

    void
    service_factory::start_services(const sstmac::hw::node::ptr& node,
        const sim_parameters::ptr& params)
    {

      if (object_map_ != 0)
        {
          std::map<std::string, SSTDesc_base*>::const_iterator it, end =
              object_map_->end();
          for (it = object_map_->begin(); it != end; ++it)
            {
              ptr_type::ptr p = it->second->build(params);
              service::ptr serv = boost::dynamic_pointer_cast<service>(p);
              node->add_service(serv);
            }
        }
    }

    service::service()
    {
      wait_ = waiter::construct();

      init_handler(ptr(this));
      init_waiter(wait_);
    }

    service::~service(){

    }

    void
    service::unregister_all_libs()
    {
        //clear self-reference
        init_handler(eventhandler::ptr());
        library::unregister_all_libs();
    }

    bool
    service::is_app_init(appid aid)
    {
      return (fullos_->env_.find(aid) == fullos_->env_.end());
    }

    void
    service::init_full_os(const boost::intrusive_ptr<operatingsystem> &o)
    {
      fullos_ = o;
    }

    }
}

