/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SUBPACKAGES_EIGER_EIGERMESSAGE_H_INCLUDED
#define SUBPACKAGES_EIGER_EIGERMESSAGE_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/ptr_type.h>

#include <string>
#include <map>

namespace sstmac {

    class eigermessage :
        public library_message
	{
		protected:
			eigermessage(const std::map<std::string, double> &input_params, const std::string &model_fname);

		public:

            typedef boost::intrusive_ptr<eigermessage> ptr;

			static eigermessage::ptr
			construct(const std::map<std::string, double> &input_params, const std::string &model_fname);

			virtual ~eigermessage() {}

            virtual std::string to_string() const {
				return "eigermessage";
			}

			std::map<std::string, double> input_params_;
			std::string model_fname_;

	};
} // end of namespace sstmac

#endif
