/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SUBPACKAGES_EIGER_EIGERNODE_H_INCLUDED
#define SUBPACKAGES_EIGER_EIGERNODE_H_INCLUDED

#include <sstmac-eiger/eigerprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/hardware/memory/memorymodel.h>
#include <sstmac/hardware/disk/vesper/disk_controller.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/hardware/node/simplenode.h>
#include <sstmac/common/nocopy.h>

#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/units.h>
#include <sstmac/common/malloc.h>

#include <map>

//#include <sstmac/processor/processor_config.h>

namespace sstmac
{
  namespace hw
  {

    /**
     * A stand-in node.  This node type passes all requests directly to
     * the simulator and has no resource contention (infinite processors),
     */
    class eigernode :
        public simplenode,
        virtual public nocopy
    {

    protected:
      eigernode(const sim_parameters::ptr &params);

      eigernode(const sim_parameters::ptr &params, nodeaddress::ptr my_id);

      sstmac::hw::eigerprocessor::ptr proc_eiger_;

    public:
      typedef boost::intrusive_ptr<eigernode> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        return new eigernode(params);
      }

      static ptr
      construct(sim_parameters::ptr params, nodeaddress::ptr my_id)
      {
        return new eigernode(params, my_id);
      }

	  virtual void
      execute_kernel(AMI::AMI_COMM_FUNC func, const boost::intrusive_ptr<sst_message>& data);

	  virtual void
      execute_kernel(AMI::AMI_COMP_FUNC func, const boost::intrusive_ptr<sst_message>& data);


			// ------- AMI functions --------- //
      virtual bool kernel_supported(AMI::AMI_COMP_FUNC) const;

			virtual node::ptr
      clone(nodeaddress::ptr my_id) const;

      virtual void
      initialize(eventmanager::ptr man);

    };

  }
} // end of namespace sstmac

#endif
