/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/common/ptr_type.h>

#include <sstmac/common/errors.h>
#include <sstmac/common/fileio.h>
#include <sstmac-eiger/eigerprocessor.h>
#include <sstmac-eiger/eigermessage.h>
#include <sstmac/common/messages/eventdata.h>
#include <sstmac/hardware/processor/instructionprocessor-config.h>

#include <iostream>

namespace sstmac
{
  namespace hw
  {
		
		/// static map pointing to all models pulled from files
		static std::map<std::string, eigermodel::ptr> model_map_;

    //
    // Constructor
    //
    eigerprocessor::eigerprocessor(
        const instructionprocessor_config::ptr& cfg,
        const eventhandler::ptr &completion, computescheduler::ptr sched) :
				instructionprocessor(cfg, completion, sched)
    {
    }

    //
    // Construct a new processor with the given average service times
    //
    eigerprocessor::ptr
    eigerprocessor::construct(const instructionprocessor_config::ptr& cfg,
        const eventhandler::ptr &completion, computescheduler::ptr sched)
    {
      eigerprocessor::ptr retval(
          new eigerprocessor(cfg, completion, sched));

      return retval;
    }

    processor::ptr
    eigerprocessor::clone() const
    {

      return construct(config_, completion_, csched_);

    }

    void
    eigerprocessor::compute(const sst_message::ptr& inst, long thread_id)
    {
			eigermessage::ptr emsg = boost::dynamic_pointer_cast<eigermessage>(inst);
			if(!emsg){
				throw valueerror("eigerprocessor: invalid message type passed to this proc.");
			}
			
			eigermodel::ptr model;
			std::map<std::string, eigermodel::ptr>::iterator it;
			if((it = model_map_.find(emsg->model_fname_)) == model_map_.end()){
				model = eigermodel::construct(emsg->model_fname_);
				model_map_.insert(std::pair<std::string, eigermodel::ptr>(emsg->model_fname_, model));
			}
			else{
				model = it->second;
			}

			timestamp compute_time = timestamp(model->poll(emsg->input_params_));
			SSTMAC_DEBUG << "eigerprocessor: computing for " << compute_time << ".\n";
			request_compute(compute_time, inst, thread_id);

			/*
			std::map<std::string,double> metrics;
			metrics["veclen"] = 38.0;
			metrics["nproc"] = 14.0;
			metrics["nrepeat"] = 8.0;
			request_compute(timestamp(model->poll(metrics)), inst, thread_id);
			*/
    }


	}
} // end of namespace sstmac

