/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SUBPACKAGES_EIGER_EIGERPROCESSOR_H_INCLUDED
#define SUBPACKAGES_EIGER_EIGERPROCESSOR_H_INCLUDED

#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor-config.h>

#include <sstmac/common/timestamp.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/common/ptr_type.h>
#include <sstmac/common/rng.h>


#include <sstmac/common/stats/power_config.h>


#include <sstmac-eiger/eigermodel.h>

namespace sstmac { namespace hw 
  {


  /// A very simple processor model

  class eigerprocessor : public instructionprocessor, virtual public nocopy
  {

	protected:

    /// Protected constructor.
    eigerprocessor(
        const instructionprocessor_config::ptr& cfg,
        const eventhandler::ptr &completion, computescheduler::ptr sched);

  public:
    typedef boost::intrusive_ptr<eigerprocessor> ptr;

    /// Goodbye.
    virtual
    ~eigerprocessor()
    {
    }

    /// Create simple processor model.
    /// \Param cfg model configuration object
    static eigerprocessor::ptr
    construct(const instructionprocessor_config::ptr& cfg,
        const eventhandler::ptr &completion, computescheduler::ptr sched);

    /// Inherited from processor.
    processor::ptr
    clone() const;

    /// Process a compute kernel.
    /// \param inst is the instructions object that will be run.
    /// \return timestamp time to process instructions.
    virtual void
    compute(const sst_message::ptr& inst, long thread_id);

    virtual std::string to_string() const {
      return "eigerprocessor";
    }

  };

  } } // end of namespace sstmac

#endif
