/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac-eiger/lib_compute_eiger.h>
#include <sstmac-eiger/eigermessage.h>

namespace sstmac
{
  namespace sw
  {

    debug lib_compute_eiger::dbg_;

    lib_compute_eiger::lib_compute_eiger(softwareid id)
    {
      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> lib_compute_eiger | lib_compute");
          logger::register_logger(&dbg_);
        }

      libname_ = "computelibeiger" + id.to_string();
    }

    lib_compute_eiger::lib_compute_eiger(std::string id)
    {
      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> lib_compute_eiger | lib_compute");
          logger::register_logger(&dbg_);
        }

      libname_ = id.c_str();
    }


    void
    lib_compute_eiger::compute_eiger(const std::map<std::string,double> &input_params, const std::string &model_fname)
    {
      if (supported())
        {

					eigermessage::ptr emsg = eigermessage::construct(input_params, model_fname);

					emsg->set_lib_name(get_lib_name());

          SSTMAC_DEBUG << "lib_compute_eiger: executing compute kernel \n";

          os_->execute_kernel(AMI::AMI_COMP_EIGER, emsg);

          SSTMAC_DEBUG << "lib_compute_eiger: waiting \n";
          wait_this(emsg);

        }
      else
        {
          throw valueerror(
              "lib_compute_eiger requires the presence of a processor model which can compute Eiger models");
        }
    }

  }
} //end of namespace sstmac
