/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SUBPACKAGES_EIGER_LIB_COMPUTE_EIGER_H_INCLUDED
#define SUBPACKAGES_EIGER_LIB_COMPUTE_EIGER_H_INCLUDED

#include <sstmac/software/libraries/compute/libcompute.h>
#include <sstmac/software/process/softwareid.h>
#include <sstmac/common/sstmac-config.h>
#include <sstmac-eiger/eigermessage.h>

#include <map>

namespace sstmac
{
  namespace sw
  {

    class lib_compute_eiger :
        virtual public libcompute
    {
    public:
      typedef boost::intrusive_ptr<lib_compute_eiger> ptr;
      // ------- constructor / boost stuff -------------//

    protected:
      lib_compute_eiger(softwareid id);
      lib_compute_eiger(std::string id);

    public:

      virtual
      ~lib_compute_eiger(){ }

      // --------------------------------------//

    public:

      void
      compute_eiger(const std::map<std::string,double> &input_params, const std::string &model_fname);

      static lib_compute_eiger::ptr
      construct(softwareid id)
      {
        return ptr(new lib_compute_eiger(id));
      }

      static lib_compute_eiger::ptr
      construct(std::string id)
      {
        return ptr(new lib_compute_eiger(id));
      }

      virtual bool
      supported() const
      {
        return os_->kernel_supported(AMI::AMI_COMP_EIGER);
      }

    protected:

      static debug dbg_;
    };

  }
} //end of namespace sstmac

#endif
