/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac-eiger/test_compute_eiger.h>
#include <sstmac/common/sstmac-config.h>
#include <sstmac/software/process/app.h>

#include <sstmac/common/fileio.h>

namespace sstmac { namespace sw
{
    SSTRegister("test_compute_eiger", app, test_compute_eiger,
                "this test kernel constructs a set of dummy parameters and makes a single call to an eiger model");
  

  test_compute_eiger::test_compute_eiger(softwareid id) :
    app(id)
  {

		lib_eiger_ = lib_compute_eiger::construct(id);
		register_lib(lib_eiger_);

  }

  // Build a new minimd object.
  //
  test_compute_eiger::ptr
  test_compute_eiger::construct(softwareid id)
  {
    test_compute_eiger::ptr retval(new test_compute_eiger(id));
    // retval->set_self(retval);
    return retval;
  }

  void
  test_compute_eiger::consume_params(const sim_parameters::ptr &params)
  {

  }

  //
  // Get a copy of this object.
  //
  app::ptr
  test_compute_eiger::clone(softwareid id)
  {
    return construct(id);
  }

  test_compute_eiger::~test_compute_eiger()
  {

  }

  void
  test_compute_eiger::skeleton_main()
  {
    SSTMAC_DEBUG << "t=" << now() << ": Hello World, i'm computer number "
                 << id_.task_.id << ", about to begin computing.\n";

		std::map<std::string,double> metrics;
		metrics["veclen"] = 8.0;
		metrics["nproc"] = 4.2;
		metrics["nrepeat"] = 3.0;

		SSTFileIO::add_path(".");
		lib_eiger_->compute_eiger(metrics, "test.model");
		//lib_eiger_->compute_eiger(metrics, "alsoworks.model");
		//lib_eiger_->compute_eiger(metrics, "missingparams.model");
		SSTMAC_DEBUG << "Done!\n";
  }

} } //end of namespace sstmac
