/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_LCOS_sync_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_LCOS_sync_H_INCLUDED

#include <sstmac-hpx/lcos/lco.h>
#include <sstmac-hpx/hpx_app.h>

namespace hpx
{
  namespace lcos
  {

    /**
     * Mutex in SST/macro is actually quite simple, since only one thread is ever
     * active at a time, there's never a chance for real resource contention.
     */
    class sync : public lco
    {

    protected:
      int count_;
      std::queue<sstmac::sw::key::ptr> waiting_;

      sync() :
        count_(0)
      {

      }

    public:
      typedef boost::intrusive_ptr<sync> ptr;

      static ptr
      construct()
      {
        return ptr(new sync());
      }

      virtual
      ~sync()
      {
      }

      void
      lock(int num = 1)
      {
        SSTMAC_DEBUG << "hpx_sync: locking with " << num << "\n";
        count_ += num;
        if (count_ > 0)
          {
            sstmac::sw::key::ptr blocker = sstmac::sw::key::construct();
            waiting_.push(blocker);
            hpx_app::hpx_block(blocker);
            // hpx_app::hpx_thread_swap();
          }

      }

      void
      unlock(int num = 1)
      {
        SSTMAC_DEBUG << "hpx_sync: unlocking with " << num << "\n";

        count_ -= num;

        if (count_ <= 0 && waiting_.size() > 0)
          {
            int waitsize = waiting_.size();
            SSTMAC_DEBUG << "unblocking " << waitsize << "\n";
            for (int i = 0; i < waitsize; i++)
              {
                sstmac::sw::key::ptr blocked = waiting_.front();
                waiting_.pop();
                hpx_app::hpx_unblock(blocked);
              }

          }
      }

    };

  }
}

#endif
