/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_TEST_MPIPINGPONG_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_TEST_MPIPINGPONG_H_INCLUDED

#include <sstmac/software/libraries/mpi/mpiapp.h>

namespace fail_launch
{


    /**
     * Basic MPI ping-pong.
     */
    class fakeapp : public sstmac::sw::mpiapp
    {
      sstmac::timestamp time_;

    protected:
      /// Hi.
      fakeapp(sstmac::sw::softwareid id) :
        sstmac::sw::mpiapp(id, to_string())
      {

      }

      fakapp(const sstmac::sim_parameters::ptr& params) : sstmac::sw::mpiapp(params) {}

    public:
      typedef boost::intrusive_ptr<fakeapp> ptr;

      /// Hello.
      static ptr
      construct(sstmac::sw::softwareid id)
      {
        return ptr(new fakeapp(id));
      }

      static ptr
      construct(const sstmac::sim_parameters::ptr& params)
      {
          return new fakeapp(params);
      }

      /// Goodbye.
      virtual
      ~fakeapp() throw () { }

      /// Get a copy.
      virtual app::ptr
      clone(sstmac::sw::softwareid id)
      {
        return fakeapp::construct(id);
      }

      /// Go.
      void
      skeleton_main();

      virtual void
      consume_params(const sstmac::sim_parameters::ptr &params);

      virtual std::string
      to_string() const
      {
        return "fakeapp";
      }
    };


} // end of namespace sstmac

#endif
