/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_stearleyallocator_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_stearleyallocator_H_INCLUDED

#include <sstmac/software/services/launch/allocation/allocationstrategy.h>

namespace sstmac
{
  namespace sw
  {

    /**
     * This is an Allocator that attempts to allocate whole racks at a time
     * for an application.
     */
    class stearleyallocator : public allocationstrategy
    {
    protected:
      stearleyallocator(long num);

    public:
      typedef boost::intrusive_ptr<stearleyallocator> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        return ptr(new stearleyallocator(params->get_long_param("node_num")));
      }



      virtual void
      allocate(int nnode_requested, node_set &allocation);

      virtual
      ~stearleyallocator() throw ()
      {
      }

      void
      nodes_good(const node_set& good);

      void
      nodes_suspect(const node_set& sus);

      void
      check_nodes();

    protected:
      node_set suspects_;
    };
  }
}
#endif
