/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_stearleylaunch_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_stearleylaunch_H_INCLUDED

#include <sstmac/software/services/launch/reallaunch.h>
#include "stearleyallocator.h"

namespace sstmac
{
  namespace sw
  {

    /**
     * A launcher that can try and figure out which nodes are failed
     */
    class stearleylaunch : public reallaunch
    {
    public:
      typedef boost::intrusive_ptr<stearleylaunch> ptr;

    protected:

      stearleylaunch();

      virtual void
      msg_kill(kill_message::ptr msg);

      virtual void
      msg_complete(complete_message::ptr msg);

      stearleyallocator::ptr salloc_;

    public:


      virtual void
      init(std::list<launchinfo::ptr> &tolaunch,
          allocationstrategy::ptr &a, indexstrategy::ptr &ind);


      virtual
      ~stearleylaunch() throw ()
      {
      }

      static stearleylaunch::ptr
      construct(sim_parameters::ptr params);

      virtual std::string
      to_string() const
      {
        return "stearleylaunch";
      }

      virtual service::ptr
      clone();

    };

  }
} // end of namespace sstmac

#endif
