/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef TUTORIALS_FTPM_FTPM_H_
#define TUTORIALS_FTPM_FTPM_H_

#include <sstmac/software/libraries/mpi/mpiapp.h>
#include <sstmac/sstmacro.h>
#include "ftpm_data.h"

namespace ftpm
{

#define ARGS_TAG 1
#define RESULT_TAG 5
#define MASTER_TAG 15
#define DONE_TAG 99

#define ALLOWED_TIMEOUTS 10

  class ftpm : public sstmac::sw::mpiapp
  {
  private:
    long nwork_;

  protected:
    ftpm(sstmac::sw::softwareid id) :
      mpiapp(id, to_string())
    {
      rng_ = RNG::SimpleCombo::construct(time(NULL));
    }

    ftpm(const sstmac::sim_parameters::ptr& params) : sstmac::sw::mpiapp(params) {}

    long
    nworkers()
    {
      return nwork_;
    }

    RNG::UniformInteger::ptr rng_;
    long work_id_;

  public:
    typedef boost::intrusive_ptr<ftpm> ptr;

    static ptr
    construct(const sstmac::sim_parameters::ptr& params)
    {
        return new ftpm(params);
    }

    virtual
    ~ftpm()
    {
    }

    virtual std::string
    to_string() const
    {
      return "ftpm base class";
    }

    void
    skeleton_main();

  protected:
    //---- failure rates ----
    double hw_temp_rate_; //hw failure, node can still be used
    double hw_perm_rate_; //hw failure, node is permanently down
    double sw_rate_; //bit flip, result is wrong
    timestamp avg_delay_; //result is delayed by this much
    double delay_rate_; // probability of a delay occuring
    //---------------------

    timestamp request_timeout_;

    int update_freq_;

    //---------------- worker functions ---------------- //

    virtual void
    w_work(int dom) = 0;

    virtual MPI_Request
    w_irecv_args(int master_rank) = 0;

    virtual void
    w_send_result(int master_rank) = 0;

    //---------------- master functions ---------------- //
    virtual void
    m_isend_args(int worker_id, int worker_rank) = 0;

    virtual MPI_Request
    m_irecv_result(int worker_id, int worker_rank) = 0;

    virtual void
    m_unpack_result(int worker_id, int worker_rank, bool bitflip) = 0;

    virtual bool
    m_done() = 0;

    virtual double
    m_get_answer_err() = 0;

    virtual std::string
    m_get_answer_string() = 0;

    virtual bool
    m_collaborate() = 0;

    virtual void
    m_generate_initial_data(int rank, int masters) = 0;
  };

}

#endif /* FTPM_H_ */
