/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef TUTORIALS_FTPM_FTPM_DATA_H
#define TUTORIALS_FTPM_FTPM_DATA_H

#include <sstmac/common/ptr_type.h>
#include <sstmac/software/libraries/mpi/sstmac_mpi.h>

namespace ftpm
{

  /**
   * This base class represents data that is passed to a worker rank, and back as a result
   */
  class ftpm_data : public sstmac::ptr_type
  {

  protected:

    ftpm_data() :
      has_fault_(false)
    {
    }

    bool has_fault_; //indicates if a bit flip has occurred somewhere on the way

  public:
    typedef boost::intrusive_ptr<ftpm_data> ptr;

    /**
     * Return number of elements.
     */
    virtual long
    count() const = 0;

    /**
     * Return size in bytes.
     */
    virtual long
    size() const = 0;


    void
    set_fault()
    {
      has_fault_ = true;
    }

    bool
    get_fault() const
    {
      return has_fault_;
    }

  };

}

#endif
